require 'jira'
require 'json'

module Speedflow
  module Plugin
    module Jira
      # Jira Client
      class Client
        include ::Speedflow::Plugin::Jira::Formatter

        # @return [Prompt] Prompt.
        attr_writer :prompt

        # @return [JIRA::Client] JIRA Client.
        attr_writer :jira_client

        # Initialize.
        #
        # config - Speedflow::Plugin::Jira::Configuration instance.
        # prompt - Speedflow::Plugin::Jira::Prompt instance.
        #
        # Examples
        #
        #    Client.new({}, Speedflow::Plugin::Jira::Prompt.new)
        #    # => <Speedflow::Plugin::Jira::Client>
        #
        # Returns nothing.
        def initialize(config, prompt)
          @config = config
          @prompt = prompt
        end

        # Public: Search issue.
        #
        # project - Project name.
        # title   - Issue title.
        #
        # Returns Hash of issue.
        def search_issue(project, title)
          safe do
            jira_client
              .Issue.jql("PROJECT = \"#{project}\" AND text ~ \"#{title}\"")
          end
        end

        # Public: Create issue.
        #
        # project_key   - Project key.
        # title         - Title (summary).
        # issue_type_id - Issue type (Fixnum).
        #
        # Returns Hash of issue.
        def create_issue(project_key, title, issue_type_id)
          safe do
            issue = jira_client.Issue.build
            data = IssueFormatter.to_create(project_key, title, issue_type_id)
            issue.save!(data)
            issue.fetch
            issue
          end
        end

        # Public: Update issue.
        #
        # issue_key     - Key of issue.
        # assignee_name - Assignee name.
        #
        # Returns Hash of issue.
        def update_issue_assignee(issue_key, assignee_name)
          safe do
            issue = jira_client.Issue.find(issue_key)
            data = IssueFormatter.to_assignee(assignee_name)
            issue.save!(data)
          end
        end

        # Public: Update issue transition.
        #
        # issue_key     - Key of issue.
        # transition_id - Transition ID.
        #
        # Returns Hash of issue.
        def update_issue_transition(issue_key, transition_id)
          safe do
            issue = jira_client.Issue.find(issue_key)
            issue.transitions.build.save!(transition: transition_id)
          end
        end

        # Public: Jira project.
        #
        # project_key - Project key.
        #
        # Returns JIRA::Resource.Project.
        def project(project_key)
          safe { jira_client.Project.find(project_key) }
        end

        # Public: Jira projects.
        #
        # Returns Hash of projects.
        def projects
          safe { jira_client.Project.all }
        end

        # Public: Jira issue transitions.
        #
        # issue_key - Issue key.
        #
        # Returns Hash of transitions.
        def issue_trans(issue_key)
          safe do
            issue = jira_client.Issue.find(issue_key, expand: 'transitions')
            issue.transitions
          end
        end

        # Public: Jira issue types.
        #
        # Returns Hash of issue types.
        def issue_types
          safe { jira_client.Issuetype.all }
        end

        # Public: Issue link.
        #
        # issue_key - Issue key.
        #
        # Returns issue link.
        def issue_link(issue_key)
          issue_link = @config.by_config('site')
          issue_link << @config.by_config('context_path')
          issue_link << 'browse/'
          issue_link << issue_key
          issue_link
        end

        # Public: Issue output formatter.
        #
        # issue - ::JIRA::Resource::Issue
        #
        # Returns Hash.
        def issue_output_format(issue)
          { 'issue' => {
            'key'     => issue.key,
            'summary' => issue.fields['summary'],
            'url'     => issue_link(issue.key) } }
        end

        # Public: Safe process JIRA action.
        #
        # Returns nothing.
        def safe
          yield
        rescue ::JIRA::HTTPError => exception
          prompt.error 'Jira errors'
          prompt.errors exception
          abort
        rescue ::URI::InvalidURIError
          prompt.error 'Invalid URL'
          abort
        end

        # Public: Jira client.
        #
        # Returns <::JIRA::Client> instance.
        def jira_client
          @jira_client ||= ::JIRA::Client.new(auth)
        end

        # Public: Auth hash for JIRA.
        #
        # Returns hash of auth.
        def auth
          {
            username:     @config.by_config('username'),
            password:     @config.by_config('password'),
            site:         @config.by_config('site'),
            context_path: @config.by_config('context_path', ''),
            auth_type:    @config.by_config('auth_type', :basic).to_sym,
            read_timeout: @config.by_config('read_timeout', 120).to_i
          }
        end

        # Public: Prompt.
        #
        # Returns <::Speedflow::Plugin::Prompt> instance.
        def prompt
          @prompt ||= ::Speedflow::Plugin::Prompt.new
        end
      end
    end
  end
end
