require 'tty-prompt'
require 'json'

module Speedflow
  module Plugin
    module Jira
      # Plugin prompt
      class Prompt
        # @return [<::Speedflow::Plugin::Prompt>] Plugin prompt.
        attr_writer :prompt

        # Public: Prompt title.
        #
        # Returns String of title.
        def title
          ask('What is the title of issue?', required: true)
        end

        # Public: Prompt project.
        #
        # projects - List of projects.
        #
        # Returns String of project key.
        def project(projects)
          prompt.select('Choose the project:') do |menu|
            projects.each do |project|
              menu.choice "#{project.name} (#{project.key})", project.key
            end
          end
        end

        # Public: Prompt issue type.
        #
        # issue_types - List of issue types.
        #
        # Returns ID of issue type.
        def issue_type(issue_types)
          prompt.select('Choose the issue type:') do |menu|
            issue_types.each do |issue_type|
              menu.choice "#{issue_type.name} (#{issue_type.id})", issue_type.id
            end
          end
        end

        # Public: Prompt transition.
        #
        # projects - List of transitions.
        #
        # Returns String of status key.
        def transition(transitions)
          prompt.select('Choose the transition:') do |menu|
            transitions.each do |transition|
              menu.choice(
                "#{transition.name} (#{transition.id})", transition.id)
            end
          end
        end

        # Public: Prompt issue.
        #
        # issues - Hash of issues from block.
        #
        # Returns String key of issue.
        def issue(&issues)
          sel_issue = prompt.select('Choose the issue:') do |menu|
            menu.choice 'Retry search!', :retry
            yield(title).each do |v|
              menu.choice "#{v.summary} (#{v.key})", v.key
            end
          end

          sel_issue = issue(&issues) if sel_issue == :retry

          sel_issue
        end

        # Public: Errors from JIRA exception.
        #
        # exception - JIRA::HTTPError.
        #
        # Returns nothing.
        def errors(exception)
          if exception.response.respond_to?('body')
            response = ::JSON.parse(exception.response.body)
            response['errors'].each { |k, v| prompt.warn "- #{k}: #{v}" }
            response['errorMessages'].each { |v| prompt.warn "- #{v}" }
          end
        end

        # Delegate
        #
        # method - Method.
        # args   - Arguments.
        # block  - Block.
        #
        # Returns wathever.
        def method_missing(method, *args, &block)
          prompt.send(method, *args, &block)
        end

        # Public: Prompt.
        #
        # Returns <::Speedflow::Plugin::Prompt> instance.
        def prompt
          @prompt ||= ::Speedflow::Plugin::Prompt.new
        end
      end
    end
  end
end
